@@ -32,13 +32,13 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
-#include <dirent.h>
 #include <gnome.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gconf/gconf-client.h>
 #include "games-frame.h"
+#include "games-files.h"
 #include "glines.h"
 
 #define KEY_DIR "/apps/glines"
@@ -49,6 +49,8 @@
 #define KEY_SAVED_FIELD "/apps/glines/saved/field"
 #define KEY_SAVED_PREVIEW "/apps/glines/saved/preview"
 
+#define NCOLORS 7
+
 GConfClient *conf_client = NULL;
 
 GtkWidget *draw_area;
@@ -56,16 +58,19 @@ static GtkWidget *app, *appbar, *pref_dialog;
 GtkWidget *next_draw_area; /* XXX Shouldn't be this much externls! */
 field_props field[FIELDSIZE * FIELDSIZE];
 GdkPixbuf *ball_pixbuf = NULL;
-GdkPixmap *surface;
+GdkPixmap *surface = NULL;
 
 GtkWidget *fast_moves_toggle_button = NULL;
 
+GList * theme_list = NULL;
+
 int active = -1;
 int target = -1;
 int inmove = 0;
 int score = 0;
 
 int move_timeout = 100;
+int animate_id = 0;
 int preview[3];
 int response;
 char * ball_filename;
@@ -91,11 +96,11 @@ static void
 load_image (gchar *fname,
 	    GdkPixbuf **pixbuf)
 {
-	gchar *tmp, *fn;
+	gchar *tmp, *fn = NULL;
 	GdkPixbuf *image;
-    
-	tmp = g_build_filename ("glines", fname, NULL);
 
+	tmp = g_build_filename ("glines", fname, NULL);
+	
 	fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
 	g_free (tmp);
 
@@ -123,16 +128,6 @@ load_image (gchar *fname,
 	*pixbuf = image;
 }
 
-static void
-help_cb (GtkWidget * widget, gpointer data)
-{
-#if 0
-  GnomeHelpMenuEntry help_entry = { "glines", "user-guide/usage.html#PREFERENCES-DIALOGBOX" };
-
-  gnome_help_display (NULL, &help_entry);
-#endif
-}
-
 static void
 draw_all_balls (GtkWidget *widget)
 {
@@ -142,14 +137,14 @@ draw_all_balls (GtkWidget *widget)
 void
 set_inmove (int i)
 {
- 	static int timeou = -1;
  	int ms;
  	
  	if (inmove != i) {
  	        inmove = i;
  	        ms = (inmove?move_timeout:100);
- 	        if(timeou != -1) gtk_timeout_remove(timeou);
- 	        timeou = gtk_timeout_add(ms, animate, (gpointer)draw_area);
+		if (animate_id)
+			g_source_remove (animate_id);
+		animate_id = g_timeout_add (ms, animate, draw_area);
  	}
 }
 
@@ -855,7 +850,9 @@ set_backgnd_color (gchar *str)
 		backgnd.name = g_strdup (str) ;
 	}
 
-	gdk_color_parse (backgnd.name, &backgnd.color);
+	if (!gdk_color_parse (backgnd.name, &backgnd.color)) {
+		gdk_color_parse ("#000000", &backgnd.color);
+	}
 
 	colormap = gtk_widget_get_colormap (draw_area);
 	gdk_colormap_alloc_color (colormap, &backgnd.color, FALSE, TRUE);
@@ -887,12 +884,11 @@ static void
 bg_color_callback (GtkWidget *widget, gpointer data)
 {
 	static char *tmp = "";
-	guint8 r, g, b, a;
+	GdkColor c;
 
-	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (widget),
-				   &r, &g, &b, &a);
+	gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &c);
 
-	tmp = g_strdup_printf ("#%02x%02x%02x", r, g, b);
+	tmp = g_strdup_printf ("#%02x%02x%02x", c.red, c.green, c.blue);
 
 	gconf_client_set_string (conf_client,
 				 KEY_BACKGROUND_COLOR, tmp, NULL);
@@ -907,55 +903,32 @@ load_theme ()
 static void
 set_selection (GtkWidget *widget, char *data)
 {
+	GList * entry;
+
+	entry = g_list_nth (theme_list,
+			    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
 	gconf_client_set_string (conf_client,
 				 KEY_BALL_THEME,
-				 data, NULL);
+				 entry->data, NULL);
 }
 
-static void
-free_str (GtkWidget *widget, char *data)
+static GtkWidget * 
+fill_menu (void)
 {
-	g_free (data);
-}
+	/* FIXME: this abuses the knowledge that a GamesFileList is really
+	 * a GList, but this is going to change. */
 
-static void
-fill_menu (GtkWidget *menu, char * mtype, gboolean bg)
-{
-	struct dirent *e;
 	gchar *dname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
 						  ("glines"), FALSE, NULL);
-	DIR *dir;
-	int itemno = 0;
-	
-	dir = opendir (dname);
 
-	if (!dir)
-		return;
-	
-	while ((e = readdir (dir)) != NULL){
-		GtkWidget *item;
-		gchar *s = g_strdup (e->d_name);
+	games_file_list_free ((GamesFileList *)theme_list);
 
-		if (! g_strrstr (e->d_name, mtype)) {
-			g_free (s);
-			continue;
-		}
-			
-		item = gtk_menu_item_new_with_label (s);
-		gtk_widget_show (GTK_WIDGET(item));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), GTK_WIDGET (item));
+	theme_list = (GList *) games_file_list_new_images (dname, NULL);
+	g_free (dname);
+	games_file_list_transform_basename ((GamesFileList *) theme_list);
+	theme_list = g_list_sort (theme_list, (GCompareFunc) g_utf8_collate);  
 
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (set_selection), s);
-		g_signal_connect (G_OBJECT (item), "destroy",
-				  G_CALLBACK (free_str), s);
-
-		if (! strcmp (ball_filename, s))
-			gtk_menu_set_active (GTK_MENU (menu), itemno);
-			  
-		itemno++;
-	}
-	closedir (dir);
+	return games_file_list_create_widget ((GamesFileList *) theme_list, ball_filename);
 }
 
 static void
@@ -971,22 +944,13 @@ set_fast_moves_callback (GtkWidget *widget, gpointer *data)
 static void
 pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 {
-	switch (response)
-		{
-		case GTK_RESPONSE_HELP:
-			help_cb (NULL, NULL);
-			break;
-		case GTK_RESPONSE_CLOSE:
-		default:
-			gtk_widget_hide (GTK_WIDGET (dialog));
-			break;
-		}      
+	gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
 static void
 game_props_callback (GtkWidget *widget, void *data)
 {
-	GtkWidget *w, *menu, *omenu, *l, *fv;
+	GtkWidget *w, *omenu, *l, *fv;
 	GtkWidget *frame;
 	GtkWidget *table;
 
@@ -1017,10 +981,9 @@ game_props_callback (GtkWidget *widget, void *data)
 			gtk_misc_set_alignment (GTK_MISC (l), 0, 0.5);
 			gtk_table_attach_defaults (GTK_TABLE (table), l, 0, 1, 0, 1);
 	    
-			omenu = gtk_option_menu_new ();
-			menu = gtk_menu_new ();
-			fill_menu (menu, ".png", FALSE);
-			gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+			omenu = fill_menu ();
+			g_signal_connect (G_OBJECT (omenu), "changed",
+					  G_CALLBACK (set_selection), NULL);
 			gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 
@@ -1029,13 +992,10 @@ game_props_callback (GtkWidget *widget, void *data)
 			gtk_table_attach_defaults (GTK_TABLE (table), l, 0, 1, 1, 2);
 	    
 			{
-				int ur, ug, ub;
-				
-				w  = gnome_color_picker_new ();
-				sscanf (backgnd.name, "#%02x%02x%02x", &ur, &ug, &ub);
-				gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (w), ur, ug, ub, 0);
+				w  = gtk_color_button_new ();
+				gtk_color_button_set_color (GTK_COLOR_BUTTON (w), &backgnd.color);
 				g_signal_connect (G_OBJECT (w), "color_set",
-						  G_CALLBACK (bg_color_callback), &backgnd.name);
+						  G_CALLBACK (bg_color_callback), NULL);
 			}
 
 			gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 1, 2);
@@ -1070,6 +1030,8 @@ game_props_callback (GtkWidget *widget, void *data)
 static int
 game_quit_callback (GtkWidget *widget, void *data)
 {
+	if (animate_id)
+		g_source_remove (animate_id);
 	gtk_main_quit ();
 	return FALSE;
 }
@@ -1163,6 +1125,14 @@ load_properties (void)
 	load_theme ();
 }
 
+static gint clamp_int (gint input, gint low, gint high)
+{
+	if (input < low)
+		input = low;
+	if (input > high)
+		input = high;
+	return input;
+}
 
 static void
 restart (void)
@@ -1170,19 +1140,25 @@ restart (void)
 	gchar *buf;
 	int i;
 
+	/* This isn't really a good idea, but if we're going to
+	 * restore the game the score has to be stored somewhere
+	 * and without some sort of restricted-access storage the
+	 * user will always be able to change it. */
 	score = gconf_client_get_int (conf_client,
                                       KEY_SAVED_SCORE, NULL);
-
+	if (score < 0)
+		score = 0;
+	
 	buf = gconf_client_get_string (conf_client,
                                        KEY_SAVED_FIELD, NULL);
 	if(buf)
 	{
 		for(i = 0; i < FIELDSIZE * FIELDSIZE; i++)
 		{
-			field[i].color = buf[i*4] - 'h';
-			field[i].pathsearch = buf[i*4 + 1] - 'h';
-			field[i].phase = buf[i*4 + 2] - 'h';
-			field[i].active = buf[i*4 + 3] - 'h';
+			field[i].color = clamp_int (buf[i*4] - 'h', 1, NCOLORS);
+			field[i].pathsearch = clamp_int (buf[i*4 + 1] - 'h', -1, FIELDSIZE*FIELDSIZE);
+			field[i].phase = clamp_int (buf[i*4 + 2] - 'h', 0, 3);
+			field[i].active = clamp_int (buf[i*4 + 3] - 'h', -1, 1);
 		}
 		g_free(buf);
 	}
@@ -1191,7 +1167,7 @@ restart (void)
 	if(buf)
 	{
 		for(i = 0; i < 3; i++)
-			preview[i] = buf[i] - 'h';
+			preview[i] = clamp_int (buf[i] - 'h', 1, NCOLORS);
 		g_free(buf);
 	}
 }
@@ -1260,18 +1236,20 @@ ball_theme_changed_cb (GConfClient *client,
 
 	theme_tmp = gconf_client_get_string (client,
 					     KEY_BALL_THEME, NULL);
-	if (strcmp (theme_tmp, ball_filename) != 0)
-		{
-			g_free (ball_filename);
-			ball_filename = theme_tmp;
-		} 
-	else
-		g_free (theme_tmp);
-  
-	load_theme ();
-	refresh_screen ();
 
-	//FIXME apply in the prefs dialog GUI
+	if (theme_tmp) {
+		if (strcmp (theme_tmp, ball_filename) != 0)
+			{
+				g_free (ball_filename);
+				ball_filename = theme_tmp;
+				load_theme ();
+				refresh_screen ();
+			} 
+		else
+			g_free (theme_tmp);
+	}
+	
+	/* FIXME apply in the prefs dialog GUI */
 }
 
 static void
@@ -1284,12 +1262,11 @@ move_timeout_changed_cb (GConfClient *client,
 
 	timeout_tmp = gconf_client_get_int (client,
 					    KEY_MOVE_TIMEOUT, NULL);
-	if ((timeout_tmp != move_timeout)
-	    && (timeout_tmp > 0)
-	    && (timeout_tmp <= 1000))
+	timeout_tmp = clamp_int (timeout_tmp, 1, 1000);
+	if (timeout_tmp != move_timeout)
 		move_timeout = timeout_tmp;
   
-	//FIXME apply in the prefs dialog GUI
+	/* FIXME apply in the prefs dialog GUI */
 }
 
 static void
@@ -1353,7 +1330,7 @@ main (int argc, char *argv [])
 	else
 		reset_game ();
 
-	app = gnome_app_new ("glines", _("Glines"));
+	app = gnome_app_new ("glines", _("Lines"));
 
 	gtk_window_set_resizable (GTK_WINDOW (app), FALSE);
 	g_signal_connect (G_OBJECT (app), "delete_event",
@@ -1404,8 +1381,6 @@ main (int argc, char *argv [])
 
 	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK);
 
-	gtk_widget_realize (draw_area);
-
         init_config ();
 	load_properties ();
 
@@ -1417,7 +1392,7 @@ main (int argc, char *argv [])
 	g_signal_connect (G_OBJECT (next_draw_area), "expose_event",
 			  G_CALLBACK (preview_expose_event), NULL);
 
-	gtk_widget_realize (next_draw_area);
+	gtk_widget_realize (next_draw_area); 
 
 	update_score_state ();
 
