@@ -38,14 +38,14 @@ typedef enum
 {
   VIEW_CURTAIN_OPEN,
   VIEW_CURTAIN_ANIMATE,
-  VIEW_CURTAIN_CLOSED,
+  VIEW_CURTAIN_CLOSED
 } ViewCurtainMode;
 
 
 typedef enum
 {
   DISPLAY_IMAGE,
-  DISPLAY_CAVE,
+  DISPLAY_CAVE
 } ViewDisplayMode;
 
 
@@ -68,14 +68,18 @@ struct _GStonesView
   ViewDisplayMode         display_mode;
 
   /* cave display */
-  guint                   x_offset;
-  guint                   y_offset;
+  gint                   x_offset;
+  gint                   y_offset;
   gboolean                x_scrolling;
   gboolean                y_scrolling;
 
   /* image display */
   GdkPixmap              *image;
 
+  /* image buffer */
+  GdkPixmap *view_buffer;
+  GdkPixmap *last_image[CAVE_MAX_WIDTH+2][CAVE_MAX_HEIGHT+2];
+ 
   /* curtain display */
   GdkPixmap              *curtain_image;
   ViewCurtainDisplayMode  curtain_display_mode;
@@ -91,7 +95,7 @@ struct _GStonesViewClass
 };
 
 
-GtkType    view_get_type         (void);
+GType    view_get_type         (void);
 
 GtkWidget *view_new              (GdkPixbuf *curtain_image);
 
@@ -104,4 +108,11 @@ void       view_display_cave     (GStonesView *view, GStonesCave *cave);
 void       view_calculate_offset (GStonesView *view, GStonesCave *cave);
 
 
+
+void       center_scroll (GStonesView *view, GStonesCave *cave);
+void       atari_scroll (GStonesView *view, GStonesCave *cave);
+void       smooth_scroll (GStonesView *view, GStonesCave *cave);
+
+
 #endif
+
