@@ -904,10 +904,19 @@ check_free (void)
 						     GTK_DIALOG_MODAL
 						     | GTK_DIALOG_DESTROY_WITH_PARENT,
 						     GTK_MESSAGE_INFO,
-						     GTK_BUTTONS_OK,
+						     GTK_BUTTONS_NONE,
 						     (_("There are no more moves.")));
+			gtk_dialog_add_buttons (GTK_DIALOG (mb),
+						_("Shuffle"),
+						GTK_RESPONSE_REJECT,
+						GTK_STOCK_OK,
+						GTK_RESPONSE_ACCEPT,
+						NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (mb),
+							 GTK_RESPONSE_ACCEPT);
 			gtk_dialog_set_has_separator (GTK_DIALOG (mb), FALSE);
-			gtk_dialog_run (GTK_DIALOG (mb));
+			if (gtk_dialog_run (GTK_DIALOG (mb)) == GTK_RESPONSE_REJECT)
+				shuffle_tiles_callback (NULL, NULL);
 			gtk_widget_destroy (mb);
                 }
                 game_over = GAME_LOST;
@@ -1576,9 +1585,12 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
         gint i, previous = 0, first=1, num_shuffle=0;
         tile temp;
         time_t seconds;
+	gboolean ok;
 
         if (paused || game_over == GAME_DEAD || game_over == GAME_WON) return;
 
+#undef USE_NEW_SHUFFLE_ALGORITHM
+#ifndef USE_NEW_SHUFFLE_ALGORITHM
         do {
                 num_shuffle++;
                 /* We do a circular permutation */
@@ -1598,8 +1610,15 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
                 tiles[previous].image = temp.image;
         }
         while (!(update_moves_left ()) && num_shuffle < visible_tiles);
-        
-        if (num_shuffle >= visible_tiles) {
+     
+#endif
+	ok = shuffle ();
+   
+#ifdef USE_NEW_SHUFFLE_ALGORITHM
+	if (!ok) {
+#else
+	if (num_shuffle >= visible_tiles) { 
+#endif
                 GtkWidget *mb;
                 game_over = GAME_DEAD;
                 games_clock_stop (GAMES_CLOCK (chrono));
